Renderer

Struct Renderer 

pub struct Renderer { /* private fields */ }
Low-level GPU renderer built on wgpu

Handles rendering pipelines, surface configuration, resources (textures, buffers), & drawing

Implementations

Source

impl Renderer

pub async fn new( window: impl Into<SurfaceTarget<'static>> + WindowHandle, ) -> Self

Creates a renderer & initializes GPU state using the window’s surface

Sets up wgpu, pipelines, default texture & camera resources

pub fn instance(&self) -> &Instance

Returns a reference to the underlying wgpu Instance

pub fn adapter(&self) -> &Adapter

Returns a reference to the underlying wgpu Adapter

pub fn device(&self) -> &Device

Returns a reference to the underlying wgpu Device

pub fn queue(&self) -> &Queue

Returns a reference to the underlying wgpu Queue

pub fn set_clear_color(&mut self, color: [f64; 4])

Sets the clear color for future render passes

pub fn begin_frame(&mut self, target: &mut dyn RenderTarget) -> Option<Frame>

Begins a frame with the given render target

pub fn end_frame(&mut self, frame: Frame)

Ends the frame by submitting commands and presenting

pub fn begin_render_pass<'a>( &'a self, encoder: &'a mut CommandEncoder, view: &'a TextureView, ) -> RenderPass<'a>

Begins a render pass with the given encoder and target view. Clears the view (set by Self::set_clear_color)

pub fn draw_batch( &self, r_pass: &mut RenderPass<'_>, batch: &mut GeometryBatch, texture_id: Option<usize>, shader_id: Option<usize>, )

Draws a geometry batch within an existing render pass

pub fn upload_camera_matrix(&mut self, view_proj: [[f32; 4]; 4])

Uploads the given view-projection matrix to the GPU for use in vertex transforms

pub fn create_offscreen_target( &self, width: u32, height: u32, format: TextureFormat, ) -> OffscreenTarget

Create an offscreen render target

pub fn add_offscreen_texture( &mut self, offscreen: &mut OffscreenTarget, ) -> usize

Adds an offscreen target texture & returns its id

pub fn add_texture(&mut self, data: &[u8]) -> usize

Adds a new texture from image bytes & returns its id

pub fn add_texture_raw(&mut self, w: u32, h: u32, data: &[u8]) -> usize

Adds a texture from raw RGBA bytes & returns its id

pub fn update_texture(&mut self, index: usize, data: &[u8])

Replaces an existing texture with new image data

pub fn update_texture_raw(&mut self, index: usize, w: u32, h: u32, data: &[u8])

Replaces an existing texture with raw RGBA bytes

pub fn add_shader(&mut self, wgsl_source: &str) -> usize

Creates a custom shader pipeline from WGSL source code Returns the pipeline index for use in draw calls

pub fn add_uniform(&mut self, data: &[u8]) -> usize

Creates a uniform buffer and returns its id

pub fn update_uniform(&mut self, id: usize, data: &[u8])

Updates an existing uniform buffer with new data

pub fn add_shader_with_uniforms( &mut self, wgsl_source: &str, uniform_ids: &[usize], ) -> usize

Creates a custom shader pipeline with associated uniform buffers Returns the pipeline index for use in draw calls